/**
 * @package PublishPress
 * @author PublishPress
 *
 * Copyright (c) 2020 PublishPress
 *
 * ------------------------------------------------------------------------------
 * Based on Edit Flow
 * Author: Daniel Bachhuber, Scott Bressler, Mohammad Jangda, Automattic, and
 * others
 * Copyright (c) 2009-2016 Mohammad Jangda, Daniel Bachhuber, et al.
 * ------------------------------------------------------------------------------
 *
 * This file is part of PublishPress
 *
 * PublishPress is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PublishPress is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PublishPress.  If not, see <http://www.gnu.org/licenses/>.
 */

@import "_elements";

.pressshack-admin-wrapper a:link,
.pressshack-admin-wrapper a:visited,
.pressshack-admin-wrapper a:active,
.pressshack-admin-wrapper a:hover {
  text-decoration: none;
}

.pressshack-admin-wrapper a,
.pressshack-admin-wrapper button,
.pressshack-admin-wrapper button::before {
  -webkit-transition: all 200ms ease-in-out;
  -moz-transition: all 200ms ease-in-out;
  -o-transition: all 200ms ease-in-out;
  transition: all 200ms ease-in-out;
}

.pressshack-admin-wrapper a,
.pressshack-admin-wrapper a div,
.pressshack-admin-wrapper a p {
  color: @main_color;
}

.pressshack-admin-wrapper a:hover,
.pressshack-admin-wrapper a:focus,
.pressshack-admin-wrapper a:active,
.pressshack-admin-wrapper a:hover div,
.pressshack-admin-wrapper a:focus div,
.pressshack-admin-wrapper a:active div,
.pressshack-admin-wrapper a:hover p,
.pressshack-admin-wrapper a:focus p,
.pressshack-admin-wrapper a:active p,
.pressshack-admin-wrapper a:hover .dashicons:before,
.pressshack-admin-wrapper a:focus .dashicons:before,
.pressshack-admin-wrapper a:active .dashicons:before {
  color: #5A4F87;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  -o-box-shadow: none;
}

.pressshack-admin-wrapper > header h1 a,
.pressshack-admin-wrapper > header h1 a:hover,
.pressshack-admin-wrapper > header h1 a:focus,
.pressshack-admin-wrapper > header h1 a:active {
  color: #23282d;
}

.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab {
  color: inherit;
}

.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab-active,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:hover,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:active,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:focus {
  border-top-color: @main_color;
  color: @main_color;
}

.pressshack-admin-wrapper > footer {
  text-align: center;
}

.pressshack-admin-wrapper > footer > div.pp-rating {
  font-size: 12px;
  margin-bottom: 10px;
  margin-top: 30px;
}

.pressshack-admin-wrapper > footer * {
  color: #777;
}

.pressshack-admin-wrapper > footer > nav ul {
  list-style: none;
}

.pressshack-admin-wrapper > footer > nav ul > li {
  display: inline-block;
}

.pressshack-admin-wrapper > footer > nav ul > li:not(:first-child) {
  margin-left: 15px;
}

.pressshack-admin-wrapper > footer > nav ul > li > a {
  font-weight: bold;
}

.pressshack-admin-wrapper > footer .dashicons.dashicons-star-filled {
  line-height: 18px;
  font-size: 12px;
  width: 12px;
  height: 12px;
  color: #FFB300;
  -webkit-transition: color 200ms ease-in-out;
  -moz-transition: color 200ms ease-in-out;
  -o-transition: color 200ms ease-in-out;
  transition: color 200ms ease-in-out;
}

.pressshack-admin-wrapper .pp-pressshack-logo img {
  width: 170px;
}

/**
 * Fremius tweaks
 */

#piframe,
.fs-secure-notice {
  display: none;
}

// Design consistency
#pp-permissions-wrapper {

  .ppButtons();
  .ppTabs();

}