@color_1: #fff;
@color_2: #a0e0ff;
@color_3: #39b6f5;
@color_4: #c00;
@color_5: #0b0;
@color_6: #333;
@color_7: white;
@color_8: #686868;
@color_9: inherit;
@color_10: #777;
@color_11: #655997;
@color_12: #aaa;
@color_13: #070;
@color_14: #8b8b8b;
@color_15: #222;
@color_16: #68589b;
@color_17: #bdbdbd;
@color_18: #1e1e1e;
@color_19: #333;
@color_20: #4caf50;
@color_21: #f44336;
@color_22: #aa2222;
@color_23: #ff5722;
@color_24: #000000;
@color_25: #181818;
@color_26: var(--text-muted);
@color_27: #6b7280;
@color_28: var(--success-color);
@color_29: var(--primary-color);
@color_30: var(--warning-color);
@color_31: #92400e;
@color_32: #3b82f6;
@color_33: #059669;
@font_family_1: courier new, courier, monospace;
@font_family_2: monospace;
@background_color_1: #000c0c;
@background_color_2: #858585;
@background_color_3: #cfebf7;
@background_color_4: #f4f4f4;
@background_color_5: #ffffe0;
@background_color_6: #fff;
@background_color_7: white;
@background_color_8: #d63638;
@background_color_9: #3f9716;
@background_color_10: #f3e5f5;
@background_color_11: #e1bee7;
@background_color_12: #68589b;
@background_color_13: #ffffff;
@background_color_14: #f5f5f5;
@background_color_15: #ee7715;
@background_color_16: #4369ff;
@background_color_17: #263dbf;
@background_color_18: #38723b;
@background_color_19: #1fb8a9;
@background_color_20: #858a8a;
@background_color_21: #ffa726;
@background_color_22: #674c0a;
@background_color_23: #e0e0e0;
@background_color_24: #e8f5e9;
@background_color_25: #ffebee;
@background_color_26: #fff3e0;
@background_color_27: #00D084;
@background_color_28: #cccccc;
@border_color_1: #e6db55;
@border_color_2: #eee;
@border_color_3: var(--cta-color);
@border_color_4: #3b82f6;
@border_color_5: #059669;
@border_left_color_1: #e0e0e0;









@keyframes pp-badge-pulse {
	0%,100% {
		transform: scale(1);
	}
	50% {
		transform: scale(1.05);
	}
}
[data-toggle="tooltip"] {
	position: relative;
	display: inline-block;
	cursor: pointer;
	.tooltip-text {
		display: none;
		position: absolute;
		min-width: 250px;
		padding: 16px;
		font-size: 13px;
		font-weight: normal;
		text-align: left;
		color: @color_1;
		background-color: @background_color_1;
		border: 1px solid #d5d5d5;
		border-radius: 10px;
		box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1), 0 0 56px rgba(0, 0, 0, 0.08);
		z-index: 99999999;
		box-sizing: border-box;
		transform: translate(-50%, -100%);
		transition: all 0.65s cubic-bezier(0.84, -0.18, 0.31, 1.26);
		white-space: normal;
		a {
			color: @color_2;
			text-decoration: underline;
			transition: color 0.2s;
			font-size: 13px !important;
			&:hover {
				color: @color_3;
			}
		}
		i {
			position: absolute;
			top: 100%;
			left: 50%;
			width: 24px;
			height: 12px;
			margin-left: -6px;
			overflow: hidden;
			&::after {
				content: '';
				position: absolute;
				width: 12px;
				height: 12px;
				left: 50%;
				background-color: @background_color_1;
				transform: translate(-50%, -50%) rotate(45deg);
				box-shadow: 0 1px 8px rgba(0, 0, 0, 0.5);
			}
		}
	}
	&:not(.click) {
		&:hover {
			.tooltip-text {
				display: block;
			}
		}
		&:focus-within {
			.tooltip-text {
				display: block;
			}
		}
	}
	&:not([data-placement]) {
		.tooltip-text {
			top: -20px;
			left: 50%;
			transform: translate(-50%, -100%);
			i {
				margin-left: -12px;
			}
		}
	}
}
[data-toggle="tooltip"].click.is-active {
	.tooltip-text {
		display: block;
	}
}
[data-toggle="tooltip"][data-placement="top"] {
	.tooltip-text {
		top: -20px;
		left: 50%;
		transform: translate(-50%, -100%);
		i {
			margin-left: -12px;
		}
	}
}
[data-toggle="tooltip"][data-placement="bottom"] {
	.tooltip-text {
		top: 100%;
		left: 25%;
		transform: translate(-50%, 0);
		margin-top: 10px;
		i {
			top: -12px;
			transform: rotate(180deg);
		}
	}
}
[data-toggle="tooltip"][data-placement="left"] {
	.tooltip-text {
		top: 50%;
		left: -10px;
		transform: translate(-100%, -50%);
		i {
			top: 50%;
			left: 100%;
			transform: translateY(-50%) rotate(-90deg);
		}
	}
}
[data-toggle="tooltip"][data-placement="right"] {
	.tooltip-text {
		top: 50%;
		left: 100%;
		transform: translate(0, -50%);
		margin-left: 10px;
		i {
			top: 50%;
			left: -12px;
			transform: translateY(-50%) rotate(90deg);
		}
	}
}
:root {
	--primary-color: #68589b;
	--primary-dark: #7b1fa2;
	--primary-light: #f3e5f5;
	--primary-lighter: #e1bee7;
	--cta-color: #feb123;
	--cta-hover: #fcca46;
	--text-primary: #1e1e1e;
	--text-secondary: #333;
	--text-muted: #6b7280;
	--success-color: #4caf50;
	--success-bg: #e8f5e9;
	--warning-color: #ff9800;
	--warning-bg: #fff3e0;
	--error-color: #f44336;
	--error-bg: #ffebee;
	--border-color: #e0e0e0;
	--background-muted: #f5f5f5;
	--background-disabled: #f5f5f5;
	--tag-builder: #ff7043;
	--tag-seo: #5c6bc0;
	--tag-ecommerce: #66bb6a;
	--tag-multilingual: #26c6da;
	--tag-community: #ffa726;
	--shadow-light: rgba(0, 0, 0, 0.05);
	--shadow-medium: rgba(0, 0, 0, 0.1);
	--shadow-dark: rgba(0, 0, 0, 0.15);
	--shadow-heavy: rgba(0, 0, 0, 0.2);
	--overlay-bg: rgba(255, 255, 255, 0.95);
}
.pp-important {
	font-weight: bold;
}
.pp-warning {
	color: @color_4;
}
.pp-success {
	color: @color_5;
}
.pp-backtan {
	background-color: @background_color_2;
}
#pp_settings_form {
	.pp-form-table {
		th {
			width: 185px !important;
			width: initial;
			padding-right: 10px;
		}
	}
	li.nav-tab {
		margin-bottom: -1px;
	}
	.activating {
		border-width: 1px;
		border-width: 1px;
		border-style: solid;
		border-style: solid;
		padding: 5px;
		padding: 5px;
		margin: 0.5em 0.5em 0 0;
		margin: 0.5em 0.5em 0 0;
		-moz-border-radius: 3px;
		-moz-border-radius: 3px;
		-khtml-border-radius: 3px;
		-khtml-border-radius: 3px;
		-webkit-border-radius: 3px;
		-webkit-border-radius: 3px;
		border-radius: 3px;
		border-radius: 3px;
		background-color: @background_color_5;
		background-color: @background_color_5;
		border-color: @border_color_1;
		border-color: @border_color_1;
		p {
			margin: 0;
			padding-top: 5px;
			padding-bottom: 5px;
		}
	}
	div.pp-subtext {
		margin-bottom: 20px !important;
	}
	&:not(.force-display-hints) {
		.pp-subtext {
			display: none;
		}
	}
	.pp-subtext-show {
		display: block;
	}
	.pp-no-hide {
		display: revert;
		margin-top: 5px;
		color: @color_9 !important;
	}
	.pp-show-hints {
		margin-left: 10px;
		opacity: 80%;
		&:hover {
			opacity: 50%;
		}
	}
	div.pp-advanced-caution {
		color: @color_10;
		font-style: italic;
		margin-top: 5px;
		padding-top: 8px;
		padding-left: 8px;
		span.pp-caution {
			font-weight: bold;
		}
		ul {
			list-style-type: disc;
			padding: 0 0 10px 25px;
			margin-top: 8px;
		}
	}
	#activation-status {
		margin-top: 20px;
	}
	#activation-error {
		margin-top: 20px;
	}
}
.pp-options {
	padding: 0 0.5em 1em 0.5em;
	border: 1px solid #ccc;
	margin-top: 0;
}
.pp-form-table {
	input {
		vertical-align: text-bottom;
	}
	tr {
		th {
			span {
				a {
					color: @color_6;
					background-color: @background_color_3;
				}
			}
			background-color: @background_color_4;
			padding-left: 10px;
			padding-right: 10px;
		}
		td {
			background-color: @background_color_4;
			padding-left: 10px;
			padding-right: 10px;
		}
		&:last-of-type {
			th {
				border-bottom: 4px solid #f4f4f4;
			}
			td {
				border-bottom: 4px solid #f4f4f4;
			}
		}
	}
}
.pp-options-table {
	tr {
		border-bottom: 1px solid #dcdcde;
		td {
			padding-top: 1em;
			padding-bottom: 1em;
		}
	}
}
.pp-options-wrapper {
	.pp-options-table {
		th {
			font-weight: bold;
			padding-top: 1em;
			padding-left: 10px;
		}
	}
}
.pp-vtight {
	input {
		margin-top: 0.15em;
	}
}
.pp-optionhint {
	margin-top: 0.5em;
	font-style: italic;
	color: @color_7;
}
table.pp-options-table {
	.pp-optionhint {
		color: @color_8;
	}
}
#wpbody-content {
	li.agp-selected_agent {
		border-bottom: none;
		a {
			font-weight: bold;
			color: @color_7;
		}
	}
}
#pp-install {
	.activating {
		border-width: 1px;
		border-style: solid;
		padding: 5px;
		margin: 0.5em 0.5em 0 0;
		-moz-border-radius: 3px;
		-khtml-border-radius: 3px;
		-webkit-border-radius: 3px;
		border-radius: 3px;
		background-color: @background_color_5;
		border-color: @border_color_1;
	}
	#edd_key {
		width: 520px;
		&::-webkit-input-placeholder {
			color: @color_12;
		}
		&::-moz-placeholder {
			color: @color_12;
			color: @color_12;
		}
		&::-ms-input-placeholder {
			color: @color_12;
		}
	}
	#activation-button {
		vertical-align: top;
	}
	.pp-key-label {
		padding-top: 5px;
		padding-left: 10px;
		padding-right: 10px;
	}
	.pp-key-active {
		font-weight: bold;
		color: @color_13;
		vertical-align: bottom;
		margin-right: 0.5em;
	}
	.pp-key-inactive {
		font-weight: bold;
		vertical-align: bottom;
		margin-right: 0.5em;
	}
	span.pp-key-warning {
		color: @color_4;
		font-weight: normal;
	}
	div.pp-key-wrap {
		white-space: nowrap;
	}
	div.pp-key-refresh {
		white-space: nowrap;
		padding-left: 10px;
		vertical-align: middle;
	}
	div.pp-key-hint {
		margin-left: 120px;
		margin-top: 10px;
	}
	div.pp-key-hint-expired {
		margin-left: 5px;
		margin-top: 5px;
	}
	.pp-install-list {
		margin: 0.5em 0 1em 0;
		li {
			padding-left: 1em;
		}
	}
	#activation-status {
		text-align: center;
		margin-bottom: 10px;
	}
	.support_data {
		div {
			margin-bottom: 6px;
			vertical-align: bottom;
		}
	}
	.pp-feature-list-caption {
		margin-top: 20px;
	}
	.pp-bullet-list {
		list-style: disc;
		padding-left: 15px;
	}
	li.pp-pro-extensions-migration-note {
		max-width: 550px;
	}
}
#tab-panel-pp {
	.activating {
		border-width: 1px;
		border-style: solid;
		padding: 5px;
		margin: 0.5em 0.5em 0 0;
		-moz-border-radius: 3px;
		-khtml-border-radius: 3px;
		-webkit-border-radius: 3px;
		border-radius: 3px;
		background-color: @background_color_5;
		border-color: @border_color_1;
		p {
			margin: 0;
			padding-top: 5px;
			padding-bottom: 5px;
		}
	}
}
#pp-modules_table {
	span.publishpress {
		color: @color_11;
		font-weight: bold;
	}
}
#pp-install_table {
	span.publishpress {
		a {
			color: @color_11;
			font-weight: bold;
			&:visited {
				color: @color_11;
				font-weight: bold;
			}
		}
	}
	h4 {
		margin-bottom: 2px;
	}
}
#pp-modules {
	table.pp-extensions {
		th {
			width: initial !important;
			min-width: 150px;
			padding-right: 20px;
			font-weight: normal;
		}
		tr {
			th {
				margin-bottom: 1px;
				vertical-align: middle;
				border-bottom: none;
				background-color: @background_color_6;
				padding: 8px;
				line-height: 8px;
				white-space: nowrap;
			}
			td {
				margin-bottom: 1px;
				vertical-align: middle;
				border-bottom: none;
				background-color: @background_color_6;
				padding: 8px;
				line-height: 15px;
				padding-left: 5px;
			}
			margin-bottom: 1px;
			vertical-align: middle;
			border-bottom: none;
		}
		margin-top: 2px;
		margin-left: 0;
	}
	span.pp-ext-info {
		color: @color_10;
		display: -webkit-box;
		-webkit-box-orient: vertical;
		-webkit-line-clamp: 2;
		overflow: hidden;
		text-overflow: ellipsis;
	}
}
#pp_config_upload_caption {
	text-indent: -10px;
	padding-left: 10px;
	margin-top: 15px;
	margin-left: 10px;
}
#pp_config_upload_wrap {
	padding-left: 22px;
}
.pp-capabilities-caption {
	margin-left: 10px;
}
.pp_cap_descripts {
	margin-left: 10px;
	margin-right: 10px;
	margin-top: 20px;
	tr {
		border-bottom: none;
		th {
			border-bottom: none;
			line-height: 8px;
		}
		td {
			border-bottom: none;
			line-height: 15px;
			margin-bottom: 1px;
			padding-left: 5px;
			background-color: @background_color_6;
		}
	}
	th.cap-name {
		text-align: center;
	}
	td.cap-name {
		width: 22%;
		text-align: center;
		font-family: @font_family_1;
	}
	tr.const-section {
		td {
			font-style: italic;
			text-align: left;
			vertical-align: bottom;
			padding-bottom: 0;
			color: @color_14;
			font-weight: bold;
		}
	}
	td.defined {
		font-weight: bold;
	}
}
div.pp-settings-code {
	max-width: 700px;
	textarea {
		margin-top: 5px;
	}
}
div.pp-user-permissions-help {
	margin-top: 20px;
	p {
		margin-bottom: 10px !important;
	}
}
.pp-instructions {
	ol {
		list-style-type: decimal;
		li {
			margin: 1em 0 1em 2em;
		}
	}
}
#pp_defined_constants {
	th {
		width: auto;
	}
}
#pp_available_constants {
	th {
		width: auto;
	}
}
#pp-import {
	div.rsu-issue {
		margin-top: 10px;
		margin-top: 10px;
		>h4 {
			background: #eee;
			color: @color_15;
			font-size: 1.2em;
			padding: 8px 10px;
			margin: 0;
			background: #eee;
			color: @color_15;
			font-size: 1.2em;
			padding: 8px 10px;
			margin: 0;
			a {
				text-decoration: none;
				display: block;
				color: @color_15;
				text-decoration: none;
				display: block;
				color: @color_15;
				&:hover {
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
				}
				&:focus {
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
				}
				&:active {
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
					text-decoration: none;
					color: @color_15;
					box-shadow: none;
					-moz-box-shadow: none;
					-webkit-box-shadow: none;
				}
			}
		}
		ul {
			margin: 0 0 20px 0;
			border-width: 0 1px;
			border-color: @border_color_2;
			border-style: solid;
			margin: 0 0 20px 0;
			border-width: 0 1px;
			border-color: @border_color_2;
			border-style: solid;
			li {
				margin: 0 0 0px 0;
				list-style: none;
				border-bottom: 1px solid #eee;
				padding: 8px 10px;
				background-color: @background_color_7;
				margin: 0 0 0px 0;
				list-style: none;
				border-bottom: 1px solid #eee;
				padding: 8px 10px;
				background-color: @background_color_7;
			}
		}
	}
	div.rsu-notes {
		ul {
			max-width: 650px;
			margin-left: 20px;
			margin-top: 5px;
			max-width: 650px;
			margin-left: 20px;
			margin-top: 5px;
			li {
				list-style: disc;
				padding-left: 10px;
				margin-left: 5px;
				list-style: disc;
				padding-left: 10px;
				margin-left: 5px;
			}
		}
	}
	input {
		padding: 5px;
		padding: 5px;
	}
	span.prev-imports {
		padding: 5px;
		padding: 5px;
	}
}
.key-display {
	display: flex;
	margin-top: 5px;
	margin-bottom: 15px;
}
.key-value {
	flex-grow: 1;
	font-family: @font_family_2;
	padding: 8px 12px;
	background: #f6f7f7;
	border: 1px solid #dcdcde;
	border-radius: 3px 0 0 3px;
	overflow-x: auto;
}
.copy-btn {
	background: #3858e9;
	color: @color_7;
	border: none;
	padding: 8px 12px;
	border-radius: 0 3px 3px 0;
	cursor: pointer;
	transition: background 0.2s;
	&:hover {
		background: #2a4bd6;
	}
}
.btn-gray {
	display: inline-block;
	padding: 8px 16px;
	background: #898989;
	color: @color_1 !important;
	text-decoration: none;
	border-radius: 3px;
	font-size: 14px;
	transition: all 0.2s;
	border: none;
	cursor: pointer;
	text-align: center;
	&:hover {
		background: #495057;
	}
}
.pp-ads-right-sidebar {
	> div {
		&:first-child {
			display: none;
		}
	}
}
body.permissions_page_presspermit-settings {
	.nav-tab {
		span.pp-integrations {
			display: inline-block;
			vertical-align: top;
			box-sizing: border-box;
			margin: 1px 0 -1px 0;
			padding: 0 5px;
			min-width: 18px;
			height: 18px;
			border-radius: 9px;
			color: @color_1;
			font-size: 11px;
			line-height: 1.6;
			text-align: center;
			z-index: 26;
		}
		span.pp-integrations-missing {
			background-color: @background_color_8;
		}
		span.pp-integrations-active {
			background-color: @background_color_9;
		}
	}
}
#pp-integrations_table {
	margin-top: 0;
	>tbody {
		>tr {
			>td {
				padding: 5px;
			}
		}
	}
}
.pp-integrations-container {
	max-width: 100%;
	margin: 5px 0 0 0;
}
.pp-pro-banner {
	background: #68589b;
	border-radius: 8px;
	padding: 0.4rem 1rem 0.4rem 1rem;
	margin-bottom: 15px;
	color: @color_7;
	display: flex;
	align-items: center;
	justify-content: space-between;
	h2 {
		font-size: 1.4rem;
		margin: 0;
		color: @color_7 !important;
		margin-bottom: 0.3rem;
		padding: 0;
		text-align: left;
	}
	p {
		opacity: 0.9;
		margin-bottom: 2px;
	}
}
#poststuff {
	.pp-pro-banner {
		background: #68589b;
		border-radius: 8px;
		padding: 0.4rem 1rem 0.4rem 1rem;
		margin-bottom: 15px;
		color: @color_7;
		display: flex;
		align-items: center;
		justify-content: space-between;
		h2 {
			font-size: 1.4rem;
			margin: 0;
			color: @color_7 !important;
			margin-bottom: 0.3rem;
			padding: 0;
			text-align: left;
		}
		p {
			opacity: 0.9;
			margin-bottom: 2px;
		}
	}
}
.pp-pro-badge-banner {
	font-size: 0.9rem;
}
.pp-category-labels {
	display: flex;
	flex-wrap: wrap;
	gap: 0.5rem;
	margin-bottom: 1rem;
}
.pp-category-label {
	padding: 0.5rem 1rem;
	background-color: @background_color_10;
	color: @color_16;
	border-radius: 20px;
	font-size: 0.85rem;
	font-weight: 500;
	cursor: pointer;
	transition: all 0.2s;
	border: 1px solid transparent;
	user-select: none;
	&:hover {
		background-color: @background_color_11;
	}
}
.pp-category-label.active {
	background-color: @background_color_12;
	color: @color_7;
	font-weight: 600;
}
#rvy-features {
	#ppr-tab-integrations {
		background-color: @background_color_4;
	}
}
.pp-integrations-grid {
	display: grid;
	grid-template-columns: repeat(auto-fit, minmax(400px, 1fr));
	gap: 1.5rem;
	margin-bottom: 30px;
}
.pp-integration-card {
	background-color: @background_color_13;
	border-radius: 8px;
	box-shadow: 0 2px 10px rgba(0, 0, 0, 0.05);
	padding: 1.1rem;
	display: flex;
	align-items: flex-start;
	position: relative;
	overflow: hidden;
	transition: transform 0.2s, box-shadow 0.2s;
	border-left: 4px solid #68589b;
	&:hover {
		transform: translateY(-2px);
		box-shadow: 0 4px 15px rgba(0, 0, 0, 0.1);
		.pp-upgrade-overlay {
			opacity: 1;
			pointer-events: auto;
		}
	}
}
.pp-integration-card.pp-disabled {
	border-left-color: @border_left_color_1;
	.pp-integration-icon {
		background-color: @background_color_14;
		&::before {
			color: @color_6;
		}
	}
	.pp-integration-title {
		color: @color_18;
	}
	.pp-integration-description {
		opacity: 0.7;
	}
	.pp-integration-features {
		opacity: 0.7;
		li {
			&:before {
				color: @color_6;
			}
		}
	}
	.pp-toggle-label {
		color: @color_19;
	}
}
.pp-integration-card.pp-available {
	border: 1px solid #68589b;
}
.pp-integration-header {
	display: flex;
	align-items: flex-start;
	margin-bottom: 15px;
}
.pp-integration-icon {
	width: 48px;
	height: 48px;
	background: #f3e5f5;
	border-radius: 8px;
	display: flex;
	align-items: center;
	justify-content: center;
	margin-right: 1.5rem;
	flex-shrink: 0;
	position: relative;
	&::before {
		font-size: 24px;
		color: @color_16;
	}

  img.emoji {
    width: 22px !important;
    height: 22px !important;
  }
}
.pp-integration-icon.acf {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/acf.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.bbpress {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/bbpress.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.buddypress {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/buddypress.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.coauthors {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/authors.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.events-calendar {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/events.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.pagebuilders {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/pagebuilder.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.peepso {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/peepso.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.relevanssi {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/relevanssi.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.woocommerce {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/woocommerce.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.wpml {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/wpml.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.yoast {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/yoast.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.beaver {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/beaver.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.breakdance {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/breakdance.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.cms_tree_view {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/tree-view.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.elementor {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/elementor.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.nested_pages {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/nested-pages.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.publishpress_statuses {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/statuses.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.searchwp {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/searchwp.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.yootheme {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/yootheme.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.litespeed {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/litespeed.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.rank-math {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/rank-math.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.w3tc {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/w3tc.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.wp-fastest-cache {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/wp-fastest-cache.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.wp-optimize {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/wp-optimize.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.wp-super-cache {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/wp-super-cache.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.all-in-one-seo {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/all-in-one-seo.svg");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.capabilities {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/capabilities.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.authors {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/authors.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.revisions {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/revisions.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.planner {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/planner.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.checklists {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/checklists.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon.taxopress {
	&::before {
		content: "";
		display: inline-block;
		width: 48px;
		height: 48px;
		background-image: url("../img/taxopress.png");
		background-size: contain;
		background-repeat: no-repeat;
		vertical-align: middle;
	}
}
.pp-integration-icon-wrap {
	min-width: 100px;
}
.pp-integration-content {
	flex: 1;
	height: 100%;
	box-sizing: border-box;
	padding-bottom: 1.1rem;
	min-height: 185px;

	&.features-only {
		min-height: 125px;
	}
}
.pp-integration-title {
	font-size: 1.1rem;
	font-weight: 600;
	margin-top: 0;
	margin-bottom: 0.5rem;
	display: flex;
	align-items: center;
	color: @color_16;
}
.pp-category-tag {
	display: inline-block;
	font-size: 0.8rem;
	padding: 0.2rem 0.3rem;
	border-radius: 4px;
	font-weight: 600;
	white-space: nowrap;
	margin-top: 10px;
	margin-right: 1.5rem;
}
.pp-tag-builder {
	background-color: @background_color_15;
	color: @color_7;
}
.pp-tag-cache {
	background-color: @background_color_16;
	color: @color_7;
}
.pp-tag-seo {
	background-color: @background_color_17;
	color: @color_7;
}
.pp-tag-ecommerce {
	background-color: @background_color_18;
	color: @color_7;
}
.pp-tag-multilingual {
	background-color: @background_color_19;
	color: @color_7;
}
.pp-tag-fields {
	background-color: @background_color_20;
	color: @color_7;
}
.pp-tag-community {
	background-color: @background_color_21;
	color: @color_7;
}
.pp-tag-admin {
	background-color: @background_color_20;
	color: @color_7;
}
.pp-tag-events {
	background-color: @background_color_22;
	color: @color_7;
}
.pp-tag-workflow {
	background-color: @background_color_12;
	color: @color_7;
}
.pp-badge {
	color: @color_7;
	font-size: 11px;
	padding: 3px 6px;
	border-radius: 4px;
	margin-left: 10px;
	font-weight: 600;
	text-transform: uppercase;
	white-space: nowrap;
}
.pp-pro-badge {
	background: #6950bd;
	background: linear-gradient(135deg, #8b5cf6, #7c3aed) !important;
	box-shadow: 0 2px 4px rgba(139, 92, 246, 0.3) !important;
}
.pp-integration-description {
	color: @color_19;
	font-size: 0.9rem;
	margin-bottom: 1rem;
	line-height: 1.5;
}
.pp-integration-features {
	font-size: 0.85rem;
	color: @color_19;
	margin-top: 0.5rem;
	display: -webkit-box;
	-webkit-box-orient: vertical;
	-webkit-line-clamp: 3;
	overflow: hidden;
	text-overflow: ellipsis;

	ul {
		list-style: none;
		padding: 0;
		margin: 0;
	}
	li {
		position: relative;
		padding-left: 1.25rem;
		margin-bottom: 0.3rem;
		&:before {
			content: '\2713';
			position: absolute;
			left: 0;
			color: @color_16;
		}
	}
}
.pp-available {
	.pp-integration-features {
		margin-bottom: 28px;
	}
}
.pp-settings-wrapper {
	position: absolute;
	bottom: 0;
	width: 100%;
	box-sizing: border-box;
	background: #fbfbfb;
	margin: 0 -1.1rem;
	line-height: 60px;
	border-top: 1px solid #dcdcdc;
	display: grid;
	grid-template-columns: 1fr 3fr;
	grid-column-gap: 20px;
	padding: 1rem;
}
.pp-settings-toggle {
	display: flex;
	align-items: center;
}
input {
	&:checked {
		&+.pp-slider {
			background-color: @background_color_27;
			&:before {
				transform: translateX(26px);
			}
		}
	}
	&:disabled {
		&+.pp-slider {
			background-color: @background_color_28;
			cursor: not-allowed;
		}
	}
}
.pp-toggle-label {
	font-size: 0.8rem;
	color: @color_19;
}
.pp-integration-status {
	display: inline-block;
	padding: 0.25rem 0.75rem;
	background-color: @background_color_24;
	color: @color_20;
	border-radius: 4px;
	font-size: 0.8rem;
	font-weight: 600;
}
.pp-integration-status.inactive {
	background-color: @background_color_25;
	color: @color_21;
}
.pp-integration-status.disabled {
	background-color: @background_color_8;
	color: @color_7;
}
.pp-upgrade-overlay {
	position: absolute;
	top: 0;
	left: 0;
	right: 0;
	bottom: 0;
	background: rgba(255, 255, 255, 0.95);
	display: flex;
	flex-direction: column;
	align-items: center;
	justify-content: center;
	text-align: center;
	padding: 20px;
	opacity: 0;
	transition: opacity 0.3s ease;
	pointer-events: none;
	backdrop-filter: blur(2px);
	h4 {
		color: @color_23;
		font-weight: 600;
		margin: 0 auto 10px auto;
		font-size: 16px;
	}
	p {
		color: @color_19;
		margin-bottom: 10px !important;
		max-width: 300px;
		line-height: 1.4;
	}
	.pp-upgrade-buttons {
		display: flex;
		flex-direction: column;
		gap: 10px;
		align-items: center;
	}
}
.pp-upgrade-btn-secondary {
	background: transparent;
	border: 1px solid #68589b;
	color: @color_16;
	padding: 8px 16px;
	border-radius: 4px;
	text-decoration: none;
	font-weight: 500;
	font-size: 13px;
	transition: all 0.2s ease;
	display: inline-block;
	&:hover {
		background: #f3e5f5;
		text-decoration: none;
		transform: translateY(-2px);
		box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2);
	}
}
.pp-upgrade-btn-primary {
	background: var(--cta-color);
	color: #414141 !important;
	padding: 8px 16px;
	border-radius: 4px;
	text-decoration: none;
	font-weight: normal;
	font-size: 14px;
	transition: all 0.2s ease;
	display: inline-block;
	&:hover {
		background: #fcca46;
		transform: translateY(-2px);
		box-shadow: 0 2px 5px rgba(0, 0, 0, 0.2);
		text-decoration: none;
	}
}
.pp-integrations-upgrade-cta {
	background: #68589b;
	border-radius: 8px;
	padding: 0;
	text-align: center;
	margin-top: 5px;
	margin-bottom: 10px;
	color: @color_7;
	h3 {
		font-size: 24px;
		margin-bottom: 15px;
		font-weight: 600;
		color: @color_7;
	}
	p {
		max-width: 600px;
		opacity: 0.9;
		line-height: 1.6;
	}
	.pp-upgrade-btn {
		background: var(--cta-color);
		color: #414141 !important;
		font-weight: normal;
		text-decoration: none;
		padding: 8px 12px;
		-webkit-border-radius: 4px;
		-moz-border-radius: 4px;
		border-radius: 4px;
		box-sizing: border-box;
		border: 1px solid #fca871;
		break-inside: avoid;
		white-space: nowrap;
		max-width: 170px;
		&:hover {
			transform: translateY(-2px);
			box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2);
			background: #fcca46;
			color: @color_25 !important;
		}
	}
}
.pp-integration-checkbox {
	position: absolute;
	top: 20px;
	right: 20px;
	z-index: 1;
	input[type="checkbox"] {
		width: 18px;
		height: 18px;
		margin: 0;
		&:disabled {
			opacity: 0.5;
			cursor: not-allowed;
		}
	}
}
.pp-hidden {
	display: none;
	display: none;
}
.pp-features-card {
	position: relative;
	width: max-content;
	&:hover {
		transform: translateY(-2px);
		box-shadow: 0 4px 15px var(--shadow-medium);
		.pp-upgrade-overlay {
			opacity: 1;
			pointer-events: auto;
		}
	}
}
.pp-feature-promo {
	.pp-feature-grid {
		display: grid;
		grid-template-columns: repeat(auto-fit, minmax(300px, 1fr));
		gap: 20px;
		margin-bottom: 24px;
	}
	.pp-feature-card {
		background: white;
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 24px;
		transition: all 0.3s ease;
		cursor: pointer;
		position: relative;
		overflow: hidden;
		&:hover {
			box-shadow: 0 8px 25px rgba(139, 92, 246, 0.15) !important;
			transform: translateY(-2px) !important;
			.pp-upgrade-overlay {
				opacity: 1 !important;
				pointer-events: auto !important;
			}
			.pp-feature-icon {
				transform: scale(1.1) !important;
			}
		}
	}
	.pp-feature-header {
		display: flex;
		align-items: center;
		gap: 12px;
		margin-bottom: 16px;
		h4 {
			margin: 0;
			color: @color_26;
			font-size: 16px;
			font-weight: 600;
		}
	}
	.pp-feature-icon {
		width: 40px;
		height: 40px;
		border-radius: 10px;
		display: flex;
		align-items: center;
		justify-content: center;
		color: @color_7;
		font-size: 18px;
		transition: transform 0.3s ease;
	}
	.pp-feature-icon.core-protection {
		background-color: @background_color_14;
	}
	.pp-feature-icon.privacy-performance {
		background-color: @background_color_14;
	}
	.pp-feature-icon.advanced-integration {
		background-color: @background_color_14;
	}
	.pp-feature-list {
		margin: 0;
		padding: 0;
		list-style: none;
		color: @color_27;
		font-size: 14px;
		line-height: 1.6;
		li {
			display: flex;
			align-items: center;
			gap: 8px;
			margin-bottom: 8px;
			&:last-child {
				margin-bottom: 0;
			}
			.check-icon {
				color: @color_28;
			}
		}
	}
	.pp-upgrade-overlay {
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		background: rgba(255, 255, 255, 0.95);
		display: flex;
		flex-direction: column;
		align-items: center;
		justify-content: center;
		text-align: center;
		padding: 20px;
		opacity: 0;
		transition: opacity 0.3s ease;
		pointer-events: none;
		backdrop-filter: blur(2px);
		h4 {
			font-weight: 600;
			margin: 0 auto 10px auto;
			font-size: 16px;
		}
		h4.core-protection {
			color: @color_29;
		}
		h4.privacy-performance {
			color: @color_29;
		}
		h4.advanced-integration {
			color: @color_29;
		}
		p {
			color: @color_27;
			margin-bottom: 15px !important;
			max-width: 200px;
			line-height: 1.4;
			font-size: 13px;
		}
		.pp-upgrade-buttons {
			display: flex;
			flex-direction: row;
			gap: 8px;
			align-items: center;
		}
	}
	.pp-upgrade-btn-primary {
		margin: 0;
		padding: 8px 16px;
		border-radius: 6px;
		text-decoration: none;
		font-weight: normal;
		font-size: 12px;
		transition: all 0.2s ease;
		display: inline-block;
		color: #414141 !important;
		&:hover {
			transform: translateY(-1px) !important;
			box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15) !important;
			text-decoration: none !important;
		}
	}
	.pp-upgrade-btn-primary.core-protection {
		background: var(--primary-color);
	}
	.pp-upgrade-btn-primary.privacy-performance {
		background: var(--primary-color);
	}
	.pp-upgrade-btn-primary.advanced-integration {
		background: var(--primary-color);
	}
	.pp-upgrade-btn-secondary {
		background: transparent;
		padding: 8px 16px;
		border-radius: 4px;
		text-decoration: none;
		font-weight: 500;
		font-size: 11px;
		transition: all 0.2s ease;
		display: inline-block;
		border: 1px solid;
		&:hover {
			text-decoration: none !important;
		}
	}
	.pp-upgrade-btn-secondary.core-protection {
		color: @color_29;
		&:hover {
			background: rgba(139, 92, 246, 0.1) !important;
		}
	}
	.pp-upgrade-btn-secondary.privacy-performance {
		color: @color_29;
		&:hover {
			background: rgba(245, 158, 11, 0.1) !important;
		}
	}
	.pp-upgrade-btn-secondary.advanced-integration {
		color: @color_29;
		&:hover {
			background: rgba(16, 185, 129, 0.1) !important;
		}
	}
	.pp-cta-section {
		background: linear-gradient(135deg, #f8fafc, #f1f5f9);
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 24px;
		text-align: center;
		h4 {
			margin: 0 0 12px 0;
			color: @color_26;
			font-size: 18px;
			font-weight: 600;
		}
		p {
			margin: 0 0 20px 0;
			color: @color_27;
			font-size: 14px;
		}
		.pp-cta-buttons {
			display: flex;
			gap: 12px;
			justify-content: center;
			align-items: center;
			flex-wrap: wrap;
			.button-primary {
				background: var(--cta-color);
				border-color: @border_color_3;
        color: #414141 !important;
				padding: 8px 24px;
				border-radius: 8px;
				font-weight: normal;
				transition: all 0.3s ease;
				&:hover {
					background: var(--cta-hover) !important;
					transform: translateY(-1px) !important;
					box-shadow: 0 4px 12px rgba(139, 92, 246, 0.25) !important;
				}
			}
			.pp-learn-more-link {
				color: @color_29;
				background: transparent;
				padding: 8px 16px;
				border-radius: 4px;
				text-decoration: none;
				font-weight: normal;
				font-size: 13px;
				transition: all 0.2s ease;
				display: inline-block;
				border: 1px solid;
				line-height: 2.15384615;
				cursor: pointer;
				white-space: nowrap;
				box-sizing: border-box;
				min-height: 30px;
				padding: 8px 24px;
				border-radius: 8px;
				&:hover {
					transform: translateY(-2px);
					box-shadow: 0 4px 15px rgba(0, 0, 0, 0.2);
				}
			}
		}
	}
}
.pp-file-access-settings, .pp-modules-settings {
	.pp-settings-header {
		background: linear-gradient(135deg, #f8fafc, #f1f5f9);
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 20px;
		margin-bottom: 24px;
		.pp-header-content {
			display: flex;
			align-items: center;
			gap: 12px;
			margin-bottom: 12px;
			.pp-header-icon {
				font-size: 24px;
			}
			h3 {
				margin: 0;
				color: @color_26;
				font-size: 18px;
				font-weight: 600;
			}
		}
		.pp-header-description {
			margin: 0;
			color: @color_27;
			font-size: 14px;
			line-height: 1.6;
		}
		.pp-mod-rewrite-warning {
			margin-top: 12px;
			padding: 12px;
			background: #fef3cd;
			border: 1px solid #f59e0b;
			border-radius: 8px;
			display: flex;
			align-items: center;
			gap: 8px;
			.warning-icon {
				color: @color_30;
				font-size: 16px;
			}
			.warning-text {
				color: @color_31;
				font-size: 14px;
				font-weight: 500;
			}
		}
	}
	.pp-settings-grid {
		display: grid;
		grid-template-columns: repeat(auto-fit, minmax(320px, 1fr));
		gap: 20px;
		margin-bottom: 24px;
	}
	.pp-setting-card {
		background: white;
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 24px;
		transition: all 0.3s ease;
		&:hover {
			box-shadow: 0 8px 25px rgba(139, 92, 246, 0.15) !important;
			transform: translateY(-2px) !important;
			.pp-setting-icon {
				transform: scale(1.1) !important;
			}
		}
	}
	.pp-setting-header {
		display: flex;
		align-items: center;
		gap: 12px;
		margin-bottom: 20px;
		h4 {
			margin: 0;
			color: @color_26;
			font-size: 16px;
			font-weight: 600;
		}
	}
	.pp-setting-icon {
		width: 40px;
		height: 40px;
		border-radius: 10px;
		display: flex;
		align-items: center;
		justify-content: center;
		color: @color_7;
		font-size: 18px;
		transition: transform 0.3s ease;
	}
	.pp-setting-icon.privacy-controls {
		background: linear-gradient(135deg, var(--primary-dark) 0%, var(--primary-color) 100%);
	}
	.pp-setting-icon.performance-options {
		background: linear-gradient(135deg, var(--success-color), #059669);
	}
	.pp-toggle-setting {
		margin-bottom: 20px;
		&:last-child {
			margin-bottom: 0;
		}
		.pp-toggle-header {
			display: flex;
			align-items: center;
			justify-content: space-between;
			margin-bottom: 8px;
			label {
				color: @color_26;
				font-weight: 500;
				font-size: 14px;
				cursor: pointer;
			}
		}
		.pp-toggle-description {
			margin: 0;
			color: @color_27;
			font-size: 13px;
			line-height: 1.5;
		}
	}
    .pp-slider {
        position: absolute;
        cursor: pointer;
        top: 0;
        left: 0;
        right: 0;
        bottom: 0;
        background-color: #ccc;
        transition: 0.4s;
        border-radius: 24px;

        &:before {
            position: absolute;
            content: "";
            height: 16px;
            width: 16px;
            left: 4px;
            bottom: 4px;
            background-color: white;
            transition: 0.4s;
            border-radius: 50%;
        }
    }
    .pp-toggle-switch {
        position: relative;
        display: inline-block;
        width: 50px;
        height: 24px;
        margin-right: 0.75rem;

        input {
            opacity: 0;
            width: 0;
            height: 0;
        }
    }
	.pp-key-management-card {
		background: white;
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 24px;
		margin-bottom: 24px;
		.pp-key-display {
			background: #f8fafc;
			border: 1px solid var(--border-color);
			border-radius: 8px;
			padding: 16px;
			margin-bottom: 16px;
			.pp-key-header {
				display: flex;
				align-items: center;
				gap: 12px;
				margin-bottom: 8px;
				.pp-key-label {
					color: @color_26;
					font-size: 14px;
					font-weight: bold;
				}
				.pp-key-input-group {
					flex: 1;
					display: flex;
					align-items: center;
					gap: 8px;
					.pp-key-code {
						background: white;
						border: 1px solid var(--border-color);
						border-radius: 4px;
						padding: 6px 12px;
						font-family: @font_family_2;
						font-size: 13px;
						color: @color_26;
						flex: 1;
					}
					.pp-copy-btn {
						background: var(--primary-color);
						color: @color_7;
						border: none;
						border-radius: 6px;
						padding: 6px 12px;
						font-size: 12px;
						cursor: pointer;
						transition: all 0.3s ease;
						&:hover {
							background: var(--primary-color) !important;
							transform: translateY(-1px) !important;
						}
					}
				}
			}
		}
		.pp-key-description {
			margin: 0 0 12px 0;
			color: @color_27;
			font-size: 14px;
			line-height: 1.6;
		}
		.pp-action-btn {
			background: white;
			border: 2px solid var(--primary-color);
			color: @color_29;
			border-radius: 8px;
			padding: 8px 16px;
			text-decoration: none;
			font-weight: 500;
			transition: all 0.3s ease;
			display: inline-flex;
			align-items: center;
			gap: 8px;
			&:hover {
				background: var(--primary-color) !important;
				color: @color_7 !important;
				transform: translateY(-1px) !important;
				box-shadow: 0 4px 12px rgba(139, 92, 246, 0.25) !important;
			}
			.pp-action-icon {
				font-size: 16px;
			}
		}
	}
	.pp-utilities-grid {
		display: grid;
		grid-template-columns: repeat(auto-fit, minmax(280px, 1fr));
		gap: 20px;
	}
	.pp-utility-card {
		background: white;
		border: 2px solid var(--border-color);
		border-radius: 12px;
		padding: 24px;
		transition: all 0.3s ease;
		&:hover {
			box-shadow: 0 8px 25px rgba(139, 92, 246, 0.15) !important;
			transform: translateY(-2px) !important;
			.pp-utility-icon {
				transform: scale(1.1) !important;
			}
		}
	}
	.pp-utility-header {
		display: flex;
		align-items: center;
		gap: 12px;
		margin-bottom: 16px;
		h4 {
			margin: 0;
			color: @color_26;
			font-size: 16px;
			font-weight: 600;
		}
	}
	.pp-utility-icon {
		width: 40px;
		height: 40px;
		border-radius: 10px;
		display: flex;
		align-items: center;
		justify-content: center;
		color: @color_7;
		font-size: 18px;
		transition: transform 0.3s ease;
	}
	.pp-utility-icon.attachments-utility {
		background: linear-gradient(135deg, #3b82f6, #2563eb);
	}
	.pp-utility-icon.nginx-integration {
		background: linear-gradient(135deg, #059669, #047857);
	}
	.pp-utility-description {
		margin: 0 0 16px 0;
		color: @color_27;
		font-size: 14px;
		line-height: 1.6;
	}
	.pp-utility-btn {
		background: white;
		border: 2px solid;
		border-radius: 8px;
		padding: 8px 16px;
		text-decoration: none;
		font-weight: 500;
		transition: all 0.3s ease;
		display: inline-flex;
		align-items: center;
		gap: 8px;
		.pp-utility-btn-icon {
			font-size: 16px;
		}
	}
	.pp-utility-btn.attachments-utility {
		border-color: @border_color_4;
		color: @color_32;
		&:hover {
			background: #3b82f6 !important;
			color: @color_7 !important;
			transform: translateY(-1px) !important;
			box-shadow: 0 4px 12px rgba(59, 130, 246, 0.25) !important;
		}
	}
	.pp-utility-btn.nginx-integration {
		border-color: @border_color_5;
		color: @color_33;
		&:hover {
			background: #059669 !important;
			color: @color_7 !important;
			transform: translateY(-1px) !important;
			box-shadow: 0 4px 12px rgba(5, 150, 105, 0.25) !important;
		}
	}
}
.pp-tab-badge {
	display: inline-block;
	vertical-align: middle;
	animation: pp-badge-pulse 2s infinite;
}
.nav-tab {
	&:hover {
		.pp-tab-badge {
			animation: none;
			transform: scale(1.1);
			transition: transform 0.2s ease;
		}
	}
}
.nav-tab-active {
	.pp-tab-badge {
		animation: none;
	}
}
@media screen and (max-width: 782px) {
	.pp-form-table {
		tr {
			th {
				border-bottom: none;
			}
		}
		div.agp-v-spaced_input {
			margin-top: 15px;
		}
		div.agp-vtight_input {
			margin-top: 8px;
		}
		#activation-button {
			margin-top: 5px;
		}
		div.pp-subtext {
			margin-bottom: 20px;
			margin-top: 5px;
		}
	}
	div.default_privacy {
		table {
			td.rlabel {
				text-align: left;
				font-weight: bold;
			}
		}
	}
	span.pp-capabilities-caption {
		margin-left: 10px;
	}
	#pp_cap_descripts {
		margin-top: 10px;
		th {
			display: none;
		}
		td {
			padding-top: 0;
			width: 100%;
			text-align: left;
		}
		td.cap-name {
			margin-bottom: 0;
			padding-top: 5px;
			font-weight: bold;
			width: 100%;
			text-align: left;
		}
	}
	#pp_defined_constants {
		margin-top: 10px;
		th {
			display: none;
		}
		td {
			padding-top: 0;
			margin-bottom: 0;
			padding-top: 5px;
			width: 100%;
			text-align: left;
		}
		td.cap-name {
			font-weight: bold;
			width: 100%;
			text-align: left;
		}
	}
	#pp_available_constants {
		margin-top: 10px;
		th {
			display: none;
		}
		td {
			padding-top: 0;
			margin-bottom: 0;
			padding-top: 5px;
			width: 100%;
			text-align: left;
		}
		td.cap-name {
			font-weight: bold;
			width: 100%;
			text-align: left;
		}
	}
}
@media (max-width: 540px) {
	.pp-pro-banner {
		flex-direction: column;
		text-align: center;
		padding: 0.2rem;
		div {
			p {
				text-align: left;
				margin-top: 10px !important;
				margin-bottom: 10px !important;
			}
		}
	}
	.pp-pro-badge-banner {
		margin-top: 1rem;
	}
	.pp-integrations-upgrade-cta {
		padding: 5px;
		h3 {
			font-size: 20px;
		}
	}
}
@media (max-width: 455px) {
	.pp-integrations-grid {
		grid-template-columns: 1fr;
	}
	.pp-integration-card {
		flex-direction: column;
	}
	.pp-integration-icon {
		margin-right: 0;
		margin-bottom: 0.4rem;
	}
	.pp-category-tag {
		margin-top: 0;
		margin-bottom: 10px;
	}
	.pp-integration-card.pp-available {
		border: 2px solid #68589b;
	}
}
@media (max-width: 768px) {
	.pp-integrations-grid {
		grid-template-columns: 1fr;
	}
	.pp-integration-card {
		flex-direction: column;
	}
	.pp-integration-icon {
		margin-right: 0;
		margin-bottom: 1rem;
	}
	.pp-pro-banner {
		flex-direction: column;
		text-align: center;
	}
	.pp-pro-badge-banner {
		margin-top: 1rem;
	}
	.pp-integrations-upgrade-cta {
		padding: 20px;
		h3 {
			font-size: 20px;
		}
	}
}
