@import "_alert";
@import "_tooltip";
@import "item-edit-tabbed";

.block-editor .editor-styles-wrapper {
    overflow-x: visible !important;
}

.pp-exceptions table {

    /* Prevent breakage by White Dot Theme */
    table-layout: auto !important;
    width: auto;
}

.pp-exceptions table, .pp-exceptions table table th, .pp-exceptions table td {
    border: none;
}

.pp-item-exceptions-ui td {
    vertical-align: middle;
}

.pp-current-item-exceptions {
    vertical-align: top;
}

.pp-current-item-exceptions select {
    min-width: 130px;
    margin-right: 10px;
    margin-left: 10px;
}

.pp-current-item-exceptions td.pp-exc-agent {
    max-width: 200px;
}

.pp-current-item-exceptions div.pp-exc-wrap {
    max-height: 325px
}

.pp-exceptions .pp-def {
    color: #000;
    font-weight: normal;
}

.pp-exceptions .pp-no {
    color: #a00;
    font-weight: normal;
}

.pp-exceptions .pp-no2 {
    color: #a00;
    font-weight: bold;
}

.pp-exceptions .pp-yes {
    color: #090;
    font-weight: normal;
}

.pp-exceptions .pp-yes2 {
    color: #0a0;
    font-weight: bold;
}


.pp-no-exceptions {
    margin: 10px;
    margin-left: 5px;
    font-weight: bold;
}

.pp-exception-actions {
    padding: 10px;
    padding-left: 6px;
}

.pp-exception-actions a {
    text-decoration: none;
}


.pp-select-exception-agents input {
    vertical-align: top;
}

.pp-select-exception-agents h4 {
    margin-top: 0;
    margin-bottom: 5px;
}

.pp-select-exception-agents button {
    padding-top: 0;
    vertical-align: top;
}

.pp-select-exception-agents button.pp_add {
    float: none;
}

.pp-select-exception-agents .pp-agents-selection tbody tr td {
    padding-right: 10px;
}

.pp-agents-selection select.pp-new-selection, .pp-agents-selection select.pp_agent_results, select.pp-search-role {
    width: 170px;
}

/*
.pp-agents-selection tbody tr td {
    padding-right: 50px;
}
*/

.pp-exc-notes {
    color: #777;
}

.pp-exc-notes div {
    margin-top: 0.3em;
}

/* Custom Statuses Publish Metabox (not required with PPS > 2.1.13) */
div.item-condition-select {
    line-height: 1.5em;
    margin-top: 3px;
}

#submitdiv .pvt-custom {
    margin-left: 16px;
}

#submitdiv .pp-postsubmit-add-privacy {
    margin-left: 10px;
}

#visibility .pp-submitbox-customize {
    margin-left: 10px;
}

#ch_post-visibility-display {
    font-weight: bold;
}

.edit-tags-php #post-body-content #normal-sortables div.postbox {
    margin-right: 15px;
}

div.pp-user-meta-search {
    margin-top: 2px;
    margin-bottom: 2px;
}

div.pp-user-meta-search input {
    vertical-align: middle;
}

.pp-agents-selection h4 {
    margin-top: 10px;
}

.pp-submitbox-customize {
    text-decoration:none;
    padding-top:6px;
}

.pp-submitbox-customize span.dashicons-groups {
    font-size:24px;
}