/* Bootstrap Alert CSS */
.alert {
  position: relative;
  padding: 1rem 1rem;
  margin-bottom: 1rem;
  border: 1px solid transparent;
  border-radius: 0.375rem;

  &-heading {
    color: inherit;
  }

  &-link {
    font-weight: 700;
  }

  &-dismissible {
    padding-right: 3rem;

    .btn-close {
      position: absolute;
      top: 0;
      right: 0;
      z-index: 2;
      padding: 1.25rem 1rem;
    }
  }

  &-primary {
    color: #084298;
    background-color: #cfe2ff;
    border-color: #b6d4fe;

    .alert-link {
      color: #06357a;
    }
  }

  &-secondary {
    color: #41464b;
    background-color: #e2e3e5;
    border-color: #d3d6d8;

    .alert-link {
      color: #34383c;
    }
  }

  &-success {
    color: #0f5132;
    background-color: #d1e7dd;
    border-color: #badbcc;

    .alert-link {
      color: #0c4128;
    }
  }

  &-info {
    color: #055160;
    background-color: #cff4fc;
    border-color: #b6effb;

    .alert-link {
      color: #04414d;
    }
  }

  &-warning {
    color: #664d03;
    background-color: #fff3cd;
    border-color: #ffecb5;

    .alert-link {
      color: #523e02;
    }
  }

  &-danger {
    color: #842029;
    background-color: #f8d7da;
    border-color: #f5c2c7;

    .alert-link {
      color: #6a1a21;
    }
  }

  &-light {
    color: #636464;
    background-color: #fefefe;
    border-color: #fdfdfe;

    .alert-link {
      color: #4f5050;
    }
  }

  &-dark {
    color: #141619;
    background-color: #d3d3d4;
    border-color: #bcbebf;

    .alert-link {
      color: #101214;
    }
  }
}