<?php return array(
    'root' => array(
        'name' => '__root__',
        'pretty_version' => 'dev-master',
        'version' => 'dev-master',
        'reference' => '3c30bda6cf82aba865655daa490d645df660052c',
        'type' => 'library',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'dev' => true,
    ),
    'versions' => array(
        '__root__' => array(
            'pretty_version' => 'dev-master',
            'version' => 'dev-master',
            'reference' => '3c30bda6cf82aba865655daa490d645df660052c',
            'type' => 'library',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'publishpress/instance-protection' => array(
            'pretty_version' => '2.1.0',
            'version' => '2.1.0.0',
            'reference' => '24289afd5ac8f1f4098a22a45cab3939c02db15f',
            'type' => 'library',
            'install_path' => __DIR__ . '/../publishpress/instance-protection',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'publishpress/pimple-pimple' => array(
            'pretty_version' => '3.5.0.10',
            'version' => '3.5.0.10',
            'reference' => 'f2784f4b1ccaf195b373a46a74cf3742e7c9f826',
            'type' => 'library',
            'install_path' => __DIR__ . '/../publishpress/pimple-pimple',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'publishpress/psr-container' => array(
            'pretty_version' => '2.0.2.1',
            'version' => '2.0.2.1',
            'reference' => '773a2510aa39a7f0ad5d5dfc78878abb939f5704',
            'type' => 'library',
            'install_path' => __DIR__ . '/../publishpress/psr-container',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'publishpress/wordpress-reviews' => array(
            'pretty_version' => '1.2.0',
            'version' => '1.2.0.0',
            'reference' => '6d1cf428aef5fc5f72c9368fbd7ce1f14d956503',
            'type' => 'library',
            'install_path' => __DIR__ . '/../publishpress/wordpress-reviews',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
        'publishpress/wordpress-version-notices' => array(
            'pretty_version' => '2.2.0',
            'version' => '2.2.0.0',
            'reference' => '174b2a35e40dfb7ff0426cc16dc8ed0888be2169',
            'type' => 'library',
            'install_path' => __DIR__ . '/../publishpress/wordpress-version-notices',
            'aliases' => array(),
            'dev_requirement' => false,
        ),
    ),
);
