<?php
/**
 * @license MIT
 *
 * Modified by PHP-FIG on 09-December-2025 using Strauss.
 * @see https://github.com/BrianHenryIE/strauss
 */

namespace PublishPress\Psr\Container;

/**
 * No entry was found in the container.
 */
interface NotFoundExceptionInterface extends ContainerExceptionInterface
{
}
