/**
 * @package PublishPress
 * @author PublishPress
 *
 * Copyright (c) 2020 PublishPress
 *
 * ------------------------------------------------------------------------------
 * Based on Edit Flow
 * Author: Daniel Bachhuber, Scott Bressler, Mohammad Jangda, Automattic, and
 * others
 * Copyright (c) 2009-2016 Mohammad Jangda, Daniel Bachhuber, et al.
 * ------------------------------------------------------------------------------
 *
 * This file is part of PublishPress
 *
 * PublishPress is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PublishPress is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with PublishPress.  If not, see <http://www.gnu.org/licenses/>.
 */
[data-toggle="tooltip"] {
  position: relative;
  display: inline-block;
  cursor: pointer;
  /* Tooltip Box */
  /* Placements */
}
[data-toggle="tooltip"] .tooltip-text {
  display: none;
  position: absolute;
  min-width: 250px;
  padding: 16px;
  font-size: 13px;
  font-weight: normal;
  text-align: left;
  color: #fff;
  background-color: #000c0c;
  border: 1px solid #d5d5d5;
  border-radius: 10px;
  box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1), 0 0 56px rgba(0, 0, 0, 0.08);
  z-index: 99999999;
  box-sizing: border-box;
  transform: translate(-50%, -100%);
  transition: all 0.65s cubic-bezier(0.84, -0.18, 0.31, 1.26);
  white-space: normal;
  /* Arrow */
}
[data-toggle="tooltip"] .tooltip-text a {
  color: #a0e0ff;
  text-decoration: underline;
  transition: color 0.2s;
  font-size: 13px !important;
}
[data-toggle="tooltip"] .tooltip-text a:hover {
  color: #39b6f5;
}
[data-toggle="tooltip"] .tooltip-text i {
  position: absolute;
  top: 100%;
  left: 50%;
  width: 24px;
  height: 12px;
  margin-left: -6px;
  overflow: hidden;
}
[data-toggle="tooltip"] .tooltip-text i::after {
  content: '';
  position: absolute;
  width: 12px;
  height: 12px;
  left: 50%;
  background-color: #000c0c;
  transform: translate(-50%, -50%) rotate(45deg);
  box-shadow: 0 1px 8px rgba(0, 0, 0, 0.5);
}
[data-toggle="tooltip"]:not(.click):hover .tooltip-text,
[data-toggle="tooltip"]:not(.click):focus-within .tooltip-text {
  display: block;
}
[data-toggle="tooltip"].click.is-active .tooltip-text {
  display: block;
}
[data-toggle="tooltip"]:not([data-placement]) .tooltip-text,
[data-toggle="tooltip"][data-placement="top"] .tooltip-text {
  top: -20px;
  left: 50%;
  transform: translate(-50%, -100%);
}
[data-toggle="tooltip"]:not([data-placement]) .tooltip-text i,
[data-toggle="tooltip"][data-placement="top"] .tooltip-text i {
  margin-left: -12px;
}
[data-toggle="tooltip"][data-placement="bottom"] .tooltip-text {
  top: 100%;
  left: 25%;
  transform: translate(-50%, 0);
  margin-top: 10px;
}
[data-toggle="tooltip"][data-placement="bottom"] .tooltip-text i {
  top: -12px;
  transform: rotate(180deg);
}
[data-toggle="tooltip"][data-placement="left"] .tooltip-text {
  top: 50%;
  left: -10px;
  transform: translate(-100%, -50%);
}
[data-toggle="tooltip"][data-placement="left"] .tooltip-text i {
  top: 50%;
  left: 100%;
  transform: translateY(-50%) rotate(-90deg);
}
[data-toggle="tooltip"][data-placement="right"] .tooltip-text {
  top: 50%;
  left: 100%;
  transform: translate(0, -50%);
  margin-left: 10px;
}
[data-toggle="tooltip"][data-placement="right"] .tooltip-text i {
  top: 50%;
  left: -12px;
  transform: translateY(-50%) rotate(90deg);
}
/* Vars */
/* Buttons */
/* Tabs */
/* Tables */
/* Alert boxes */
.pressshack-admin-wrapper a:link,
.pressshack-admin-wrapper a:visited,
.pressshack-admin-wrapper a:active,
.pressshack-admin-wrapper a:hover {
  text-decoration: none;
}
.pressshack-admin-wrapper a,
.pressshack-admin-wrapper button,
.pressshack-admin-wrapper button::before {
  -webkit-transition: all 200ms ease-in-out;
  -moz-transition: all 200ms ease-in-out;
  -o-transition: all 200ms ease-in-out;
  transition: all 200ms ease-in-out;
}
.pressshack-admin-wrapper a,
.pressshack-admin-wrapper a div,
.pressshack-admin-wrapper a p {
  color: #655997;
}
.pressshack-admin-wrapper a:hover,
.pressshack-admin-wrapper a:focus,
.pressshack-admin-wrapper a:active,
.pressshack-admin-wrapper a:hover div,
.pressshack-admin-wrapper a:focus div,
.pressshack-admin-wrapper a:active div,
.pressshack-admin-wrapper a:hover p,
.pressshack-admin-wrapper a:focus p,
.pressshack-admin-wrapper a:active p,
.pressshack-admin-wrapper a:hover .dashicons:before,
.pressshack-admin-wrapper a:focus .dashicons:before,
.pressshack-admin-wrapper a:active .dashicons:before {
  color: #5A4F87;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  -o-box-shadow: none;
}
.pressshack-admin-wrapper > header h1 a,
.pressshack-admin-wrapper > header h1 a:hover,
.pressshack-admin-wrapper > header h1 a:focus,
.pressshack-admin-wrapper > header h1 a:active {
  color: #23282d;
}
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab {
  color: inherit;
}
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab-active,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:hover,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:active,
.pressshack-admin-wrapper .nav-tab-wrapper .nav-tab:focus {
  border-top-color: #655997;
  color: #655997;
}
.pressshack-admin-wrapper > footer {
  text-align: center;
}
.pressshack-admin-wrapper > footer > div.pp-rating {
  font-size: 12px;
  margin-bottom: 10px;
  margin-top: 30px;
}
.pressshack-admin-wrapper > footer * {
  color: #777;
}
.pressshack-admin-wrapper > footer > nav ul {
  list-style: none;
}
.pressshack-admin-wrapper > footer > nav ul > li {
  display: inline-block;
}
.pressshack-admin-wrapper > footer > nav ul > li:not(:first-child) {
  margin-left: 15px;
}
.pressshack-admin-wrapper > footer > nav ul > li > a {
  font-weight: bold;
}
.pressshack-admin-wrapper > footer .dashicons.dashicons-star-filled {
  line-height: 18px;
  font-size: 12px;
  width: 12px;
  height: 12px;
  color: #FFB300;
  -webkit-transition: color 200ms ease-in-out;
  -moz-transition: color 200ms ease-in-out;
  -o-transition: color 200ms ease-in-out;
  transition: color 200ms ease-in-out;
}
.pressshack-admin-wrapper .pp-pressshack-logo img {
  width: 170px;
}
/**
 * Fremius tweaks
 */
#piframe,
.fs-secure-notice {
  display: none;
}
#pp-permissions-wrapper .pp-default-button {
  border-color: #655997;
  color: #655997;
  border-width: 1px;
}
#pp-permissions-wrapper .pp-default-button:focus {
  color: #483f6b;
  border-color: #483f6b;
  box-shadow: 0 0 0 1px #483f6b;
  -moz-box-shadow: 0 0 0 1px #483f6b;
  -webkit-box-shadow: 0 0 0 1px #483f6b;
}
#pp-permissions-wrapper .pp-default-button:hover,
#pp-permissions-wrapper .pp-default-button:active {
  color: #483f6b;
  border-color: #483f6b;
}
#pp-permissions-wrapper .pp-primary-button {
  background: #655997;
  color: #fff;
  border-color: #655997;
  border-width: 1px;
}
#pp-permissions-wrapper .pp-primary-button:focus {
  background: #483f6b;
  color: #fff;
  border-color: #483f6b;
  box-shadow: 0 0 0 1px #fff, 0 0 0 3px #483f6b;
}
#pp-permissions-wrapper .pp-primary-button:hover,
#pp-permissions-wrapper .pp-primary-button:active {
  background: #483f6b;
  color: #fff;
  border-color: #483f6b;
}
#pp-permissions-wrapper .pp-pro-button {
  background: #feb123;
  color: #222;
  border-color: #feb123;
  border-width: 1px;
}
#pp-permissions-wrapper .pp-pro-button:focus {
  background: #febe47;
  color: #222;
  border-color: #febe47;
  box-shadow: 0 0 0 1px #fff, 0 0 0 3px #febe47;
  -moz-box-shadow: 0 0 0 1px #fff, 0 0 0 3px #febe47;
  -webkit-box-shadow: 0 0 0 1px #fff, 0 0 0 3px #febe47;
}
#pp-permissions-wrapper .pp-pro-button:hover,
#pp-permissions-wrapper .pp-pro-button:active {
  background: #febe47;
  color: #222;
  border-color: #febe47;
}
#pp-permissions-wrapper ul.pp-list_horiz {
  width: 100%;
  clear: both;
  float: left;
  margin: 0 0 -1px 0;
  padding: 0;
}
#pp-permissions-wrapper ul.pp-list_horiz li {
  list-style: none;
  float: left;
  margin: 0;
}
#pp-permissions-wrapper ul.pp-list_horiz li:hover,
#pp-permissions-wrapper ul.pp-list_horiz li:focus,
#pp-permissions-wrapper ul.pp-list_horiz li:active {
  outline: none;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-agent {
  margin: 0 -1px 0 0;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-agent a {
  text-decoration: none;
  color: #666;
  padding: 10px 14px 10px;
  display: block;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-agent a:hover,
#pp-permissions-wrapper ul.pp-list_horiz li.agp-agent a:focus,
#pp-permissions-wrapper ul.pp-list_horiz li.agp-agent a:active {
  text-decoration: none;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
  color: #222;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-selected_agent {
  border: 1px solid #ccc;
  background-color: #fff;
  border-bottom: 1px solid #fff;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-selected_agent a {
  color: #222;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-unselected_agent {
  border: 1px solid #ccc;
  background: #f1f1f1;
}
#pp-permissions-wrapper ul.pp-list_horiz li.agp-unselected_agent a {
  color: #666;
}
#pp-permissions-wrapper .pp-agents {
  background-color: #fff;
  padding: 0 0 0.2em 0.3em;
  border: 1px solid #ccc;
  margin: 0;
  padding: 20px;
}
#pp-permissions-wrapper .pp-agents a {
  color: #655997;
  text-decoration: none;
}
#pp-permissions-wrapper .pp-agents a:hover #pp-permissions-wrapper .pp-agents a:focus,
#pp-permissions-wrapper .pp-agents a:focus {
  color: #483f6b;
  text-decoration: none;
}
#pp-permissions-wrapper .pp-agents input {
  vertical-align: text-bottom;
}
