.pp-agents_caption {
    margin: 0 0 0 0.75em;
    clear: both;
}

.pp_agents_ui_wrapper {
    margin: 0.3em;
}

.pp-agents-list > li {
    white-space: nowrap;
}

.pp-agents-list_auto {
    margin: 0.5em 0 1em 0;
    padding: 0;
    clear: both;
    float: left;
    width: 100%;
}

.pp-agents-list_auto li {
    list-style: none;
    float: left;
    margin: 0.3em 1.5em 0.3em 0;
}

.pp-agents-list_auto input {
    margin: 0;
    font-size: auto;
    padding: 0;
}

.pp-agents_emsized {
    clear: both;
    width: 100%;
}

.pp-agents_emsized ul {
    margin: 0;
    padding: 0;
    clear: both;
    float: left;
    width: 100%;
}

.pp-agents_emsized ul li {
    margin: 0.3em 0 0.3em 0;
    list-style: none;
}

.pp-agents_emsized ul input {
    margin: 0 5px 0 0;
    font-size: auto;
    padding: 0;
}

.pp-agents-list_7 li {
    width: 7em;
}

.pp-agents-list_8 li {
    width: 8em;
}

.pp-agents-list_9 li {
    width: 9em;
}

.pp-agents-list_10 li {
    width: 10em;
}

.pp-agents-list_11 li {
    width: 11em;
}

.pp-agents-list_12 li {
    width: 12em;
}

.pp-agents-list_13 li {
    width: 13em;
}

.pp-agents-list_14 li {
    width: 14em;
}

.pp-agents-list_15 li {
    width: 15em;
}

.pp-agents-list_16 li {
    width: 16em;
}

.pp-agents-list_17 li {
    width: 17em;
}

.pp-agents-list_18 li {
    width: 18em;
}

.pp-agents-list_19 li {
    width: 19em;
}

.pp-agents-list_20 li {
    width: 20em;
}

.pp-agents-list_21 li {
    width: 21em;
}

.pp-agents-list_22 li {
    width: 22em;
}

.pp-agents-list_23 li {
    width: 23em;
}

.pp-agents-list_24 li {
    width: 24em;
}

.pp-agents-list_25 li {
    width: 25em;
}

.pp-agents-list_26 li {
    width: 26em;
}

.pp-agents-list_27 li {
    width: 27em;
}

.pp-agents-list_28 li {
    width: 28em;
}

.pp-agents-list_29 li {
    width: 29em;
}

.pp-agents-list_30 li {
    width: 30em;
}
